#region License
// Copyright (c) 2007 James Newton-King
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Utilities.Email;
using System.Net.Mail;
using Utilities.Testing;

namespace Utilities.Test.Email
{
  public class EmailUtilsFixture : TestFixtureBase
  {
    [Test]
    public void ParseEmailAddresses()
    {
      IList<MailAddress> emailAddresses;

      emailAddresses = EmailUtils.ParseEmailAddresses("<mail1@host1.com>, <mail2@host2.com>");
      Assert.AreEqual(emailAddresses.Count, 2);
      Assert.AreEqual(emailAddresses[0].Address, "mail1@host1.com");
      Assert.AreEqual(emailAddresses[0].DisplayName, string.Empty);
      Assert.AreEqual(emailAddresses[1].Address, "mail2@host2.com");

      emailAddresses = EmailUtils.ParseEmailAddresses(string.Empty);
      Assert.AreEqual(emailAddresses.Count, 0);

      emailAddresses = EmailUtils.ParseEmailAddresses(null);
      Assert.AreEqual(emailAddresses.Count, 0);
    }

    [Test]
    [ExpectedException(typeof(FormatException), ExpectedMessage = "Error parsing email addresses: I like cake.")]
    public void ParseEmailAddressesBadFormat()
    {
      EmailUtils.ParseEmailAddresses("I like cake.");
    }
  }
}
